package de.fitko.fitconnect.client;

import com.typesafe.config.Config;
import com.typesafe.config.ConfigBeanFactory;
import com.typesafe.config.ConfigFactory;
import de.fitko.fitconnect.api.services.Sender;
import de.fitko.fitconnect.api.services.Subscriber;
import de.fitko.fitconnect.api.services.auth.OAuthService;
import de.fitko.fitconnect.api.services.crypto.CryptoService;
import de.fitko.fitconnect.api.services.metadata.MetadataService;
import de.fitko.fitconnect.api.services.validation.CertificateValidator;
import de.fitko.fitconnect.api.services.validation.MetadataValidator;
import de.fitko.fitconnect.dependency.ApplicationConfig;
import de.fitko.fitconnect.impl.SubmissionSender;
import de.fitko.fitconnect.impl.SubmissionSubscriber;
import de.fitko.fitconnect.impl.auth.DefaultOAuthService;
import de.fitko.fitconnect.impl.crypto.JWECryptoService;
import de.fitko.fitconnect.impl.http.ProxyConfig;
import de.fitko.fitconnect.impl.metadata.MetadataUploadService;
import de.fitko.fitconnect.impl.validation.KeyValidator;
import de.fitko.fitconnect.impl.validation.MetadataSubmissionValidator;
import org.springframework.web.client.RestTemplate;

import java.io.File;

public class ClientFactory {

    public static final String CONFIG_FILENAME = "sdk.conf";
    public static final String CONFIG_CONTEXT = "sdk";

    private ClientFactory() {}

    public static SenderClient.WithDestination senderClient() {
        final ApplicationConfig config = loadConfig();
        final RestTemplate restTemplate = getRestTemplate(config);
        final Sender sender = getSender(config, restTemplate);
        return SenderClient.build(sender, config.getClientId(), config.getClientSecret());
    }

    public static SubscriberClient.RequestSubmission subscriberClient() {
        final ApplicationConfig config = loadConfig();
        final RestTemplate restTemplate = getRestTemplate(config);
        final Subscriber subscriber = getSubscriber(config, restTemplate);
        return SubscriberClient.builder(subscriber);
    }

    private static Subscriber getSubscriber(ApplicationConfig config, RestTemplate restTemplate) {
        final OAuthService oAuthService = getOAuthService(config, restTemplate);
        final CryptoService cryptoService = getCryptoService();
        final MetadataValidator validator = new MetadataSubmissionValidator();
        return new SubmissionSubscriber(oAuthService,cryptoService, validator);
    }

    private static Sender getSender(ApplicationConfig config, RestTemplate restTemplate) {
        final OAuthService authService = getOAuthService(config, restTemplate);
        final CryptoService cryptoService = getCryptoService();
        final CertificateValidator validator = new KeyValidator();
        final MetadataService metadataService = new MetadataUploadService(restTemplate);
        return new SubmissionSender(authService, cryptoService, validator, metadataService);
    }

    private static CryptoService getCryptoService() {
        return new JWECryptoService();
    }

    private static OAuthService getOAuthService(ApplicationConfig config, RestTemplate restTemplate) {
        return new DefaultOAuthService(restTemplate, config.getAuthTokenUrl());
    }

    private static RestTemplate getRestTemplate(ApplicationConfig config) {
        final ProxyConfig proxyConfig = new ProxyConfig(config.getHttpProxyHost(), config.getHttpProxyPort());
        return proxyConfig.proxyRestTemplate();
    }

    private static ApplicationConfig loadConfig() {
        final Config configFile = ConfigFactory.parseFile(new File(CONFIG_FILENAME));
        final Config sdkConfig = ConfigFactory.load(configFile).getConfig(CONFIG_CONTEXT);
        return ConfigBeanFactory.create(sdkConfig, ApplicationConfig.class);
    }
}
